// AP(r) Computer Science Marine Biology Simulation:  Class
// added by Georgia Tech to extend past Fish
//
// This class is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation.
//
// This class is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

import java.util.Random;
import java.awt.Color;
import java.util.ArrayList;

/**
 * Class that represents an abstract actionable
 * object (something that can act).
 * 
 * This class was created
 * to show how to extend the Marine Biology Case
 * Study past Fish. 
 * 
 * @author Barbara Ericson ericson@cc.gatech.edu
 * Georgia Tech Jan 2005
 */
public abstract class AbstractActionable extends AbstractDrawable 
        implements Actionable
{
  //////////////// fields //////////////////////////
  
  //////////////// constructors ////////////////////
  
  /**
   * Constructor that takes an environment and a location
   * @param theId the unique ID for this object
   * @param theEnv  the environment this object is in
   * @param theLoc  the location of this object
   */
  public AbstractActionable(int theId,
                            Environment theEnv, 
                            Location theLoc)
  {
    // call the parent constructor with a random direction
    super(theId, theEnv,theLoc);
  }
  
  /**
   * Constructor that takes an environment, location, and 
   * direction
   * @param theId the unique ID for this object
   * @param theEnv  the environment this object is in
   * @param theLoc  the location of this object
   * @param theDir the direction this object is facing
   */
  public AbstractActionable(int theId,
                            Environment theEnv, 
                            Location theLoc, 
                            Direction theDir)
  {
    super(theId, theEnv, theLoc, theDir);
  }
  
  /**
   * Constructor that takes an environment, location, and 
   * direction
   * @param theId the unique ID for this object
   * @param theEnv  the environment this object is in
   * @param theLoc  the location of this object
   * @param theDir the direction this object is facing
   * @param theColor the new color
   */
  public AbstractActionable(int theId,
                            Environment theEnv, 
                            Location theLoc, 
                            Direction theDir,
                            Color theColor)
  {
    super(theId,theEnv,theLoc,theDir,theColor);
  }
  
  //////////////// methods /////////////////////////
  
  /**
   * Method to tell the object to act
   */
  abstract public void act();
  
  /** 
   * Modifies this object's location and notifies 
   * the environment.
   * @param  newLoc    new location value
   **/
  protected void changeLocation(Location newLoc)
  {
    // Change location and notify the environment.
    Location oldLoc = location();
    this.setLocation(newLoc);
    environment().recordMove(this, oldLoc);
    
    // object is again at location loc in environment
  }
  
  /**
   * Method to return the location in the specified
   * direction if it is a neighbor and is empty
   * @return the location or null
   */
  public Location getEmptyNeighbor(Direction dir)
  {
    // declare a variable to return
    Location returnLoc = null;
    
    // get the allowed neighboring locations
    ArrayList nbrs = environment().neighborsOf(location());
    
    /* get the location in the specified direction
     * (this may not be a valid neighbor)
     */
    Location testLoc = environment().getNeighbor(location(), dir);
    
    /* if the location is empty and it is a valid neighbor
     * set the return variable
     */
    if (environment().objectAt(testLoc) == null &&        
        nbrs.contains(testLoc))
      returnLoc = testLoc;
    
    // return the location (if one else null)
    return returnLoc;
  }
  
  /**
   * Method to get the east and west empty neighbors
   * @return an array with the empty locations in it
   * there may be nothing in this array (if no
   * empty neighboring locations)
   */
  public Location[] getEastAndWestEmptyNeighbors()
  {
    Location[] locArray = new Location[2];
    int count = 0;
    Location eastLoc = getEmptyNeighbor(Direction.EAST);
    if (eastLoc != null)
    {
      locArray[count++] = eastLoc;
      Debug.println("there is a east empty loc");
    }
    Location westLoc = getEmptyNeighbor(Direction.WEST);
    if (westLoc != null)
    {
      locArray[count++] = westLoc;
      Debug.println("there is a west empty loc");
    }
    return locArray;
  }
  
  
  /**
   * Method to get the the empty neighbors to the north
   * of the current location (north, northeast, northwest)
   * @return an array with the empty locations in it
   * there may be nothing in this array (if no
   * empty neighboring locations)
   */
  public Location[] getNorthEmptyNeighbors()
  {
    Debug.println("Current Location " + this.location());
    Location[] locArray = new Location[3];
    int count = 0;
    Location loc1 = getEmptyNeighbor(Direction.NORTHEAST);
    if (loc1 != null)
    {
      locArray[count++] = loc1;
      Debug.println("northeast loc is " + loc1);
    }
    Location loc2 = getEmptyNeighbor(Direction.NORTHWEST);
    if (loc2 != null)
      locArray[count++] = loc2;
    Location loc3 = getEmptyNeighbor(Direction.NORTH);
    if (loc3 != null)
    {
      locArray[count++] = loc3;
      Debug.println("North loc is " + loc3);
    }
    return locArray;
  }
  
  /**
   * Method to get the the empty neighbors to the south
   * of the current location (south, southeast, southwest)
   * @return an array with the empty locations in it
   * there may be nothing in this array (if no
   * empty neighboring locations)
   */
  public Location[] getSouthEmptyNeighbors()
  {
    Location[] locArray = new Location[3];
    int count = 0;
    Location loc1 = getEmptyNeighbor(Direction.SOUTHEAST);
    if (loc1 != null)
      locArray[count++] = loc1;
    Location loc2 = getEmptyNeighbor(Direction.SOUTHWEST);
    if (loc2 != null)
      locArray[count++] = loc2;
    Location loc3 = getEmptyNeighbor(Direction.SOUTH);
    if (loc3 != null)
      locArray[count++] = loc3;
    return locArray;
  }
  
  /**
   * Method to move this object to the new location
   * @param nextLoc the new location
   */
  protected void move(Location nextLoc)
  {
    // if new location move
    if (!nextLoc.equals(location()))
    {
      // Move to new location.
      Location oldLoc = location();
      changeLocation(nextLoc);
      
      // Update direction in case fish had to turn to move.
      Direction newDir = 
        environment().getDirection(oldLoc, nextLoc);
      changeDirection(newDir);
      
    } 
  }
  
  
    /** Finds locations with objects in them adjacent to this fish.
     *  @return    an ArrayList containing the locations with neighbors
     **/
    protected ArrayList fullNeighbors()
    {
        // Get all the neighbors of this fish, empty or not.
        ArrayList nbrs = environment().neighborsOf(location());

        // Figure out which neighbors are not empty and add those to a new list.
        ArrayList fullLocations = new ArrayList();
        for ( int index = 0; index < nbrs.size(); index++ )
        {
            Location loc = (Location) nbrs.get(index);
            if ( !environment().isEmpty(loc) )
                fullLocations.add(loc);
        }

        return fullLocations;
    }

    /** Finds empty locations adjacent to this fish.
     *  @return    an ArrayList containing neighboring empty locations
     **/
    protected ArrayList emptyNeighbors()
    {
        // Get all the neighbors of this fish, empty or not.
        ArrayList nbrs = environment().neighborsOf(location());

        // Figure out which neighbors are empty and add those to a new list.
        ArrayList emptyNbrs = new ArrayList();
        for ( int index = 0; index < nbrs.size(); index++ )
        {
            Location loc = (Location) nbrs.get(index);
            if ( environment().isEmpty(loc) )
                emptyNbrs.add(loc);
        }

        return emptyNbrs;
    }
  
  /**
     * Method to have a fish die and remove itself
     * from the environment 
     */
    public void die() 
    {
      environment().remove(this);
    }
}
