// AP(r) Computer Science Marine Biology Simulation:
//
// This class is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation.
//
// This class is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

import java.util.Random;
import java.awt.Color;
import java.util.ArrayList;

/**
 * Class that represents a mouse.  These
 * animals like to find the exit and leave.
 * The exit is at the top right corner of the
 * environment.
 * 
 * This class was created
 * to show how to extend the Marine Biology Case
 * Study past Fish. 
 * 
 * @author Barbara Ericson ericson@cc.gatech.edu
 * Georgia Tech Jan 2005
 */
public class Mouse extends AbstractActionable
{
  //////////////// fields //////////////////////////
  
  // Class Variable: Shared among ALL Rats
  private static int nextAvailableID = 1;   // next avail id
  
  // object variable: the number of steps to exit
  private int numSteps = 0;
  
  //////////////// constructors ////////////////////
  
  /**
   * Constructor that takes an environment and a location
   * @param theEnv  the environment this object is in
   * @param theLoc  the location of this object
   */
  public Mouse(Environment theEnv, Location theLoc)
  {
    // call the other constructor with a random direction
    this(theEnv,theLoc,theEnv.randomDirection());
  }
  
  /**
   * Constructor that takes an environment, location, and 
   * direction
   * @param theEnv  the environment this object is in
   * @param theLoc  the location of this object
   * @param theDir the direction this object is facing
   */
  public Mouse(Environment theEnv, Location theLoc, 
                  Direction theDir)
  {
    // let the parent constructor handle this
    super(nextAvailableID,theEnv,theLoc,theDir);
    nextAvailableID++; // increment the next id
  }
  
  /**
   * Constructor that takes the environment, 
   * location, direction and color
   * @param theEnv the environment to use
   * @param theLoc the location to use
   * @param theDir the direction to use
   * @param theColor the color to use
   */
  public Mouse(Environment theEnv, Location theLoc, 
                  Direction theDir, Color theColor)
  {
    // let the parent constructor handle this
    super(nextAvailableID,theEnv,theLoc,theDir, theColor);
    nextAvailableID++; // increment the next id
  }
  
  //////////////// methods /////////////////////////
  
  /**
   * Method to tell the object to act
   */
  public void act()
  {
    // check that this object is still in the environment
    if (isInEnv())
    {
      // increment the number of steps
      numSteps++;
      
      // try to move
      move();
    }
  }
  
  /** Finds this animals next location.
     *  A mouse will just randomlly pick
     * an empty neighboring location to move ot.
     *  @return    the next location for this fish
     **/
    protected Location nextLocation()
    {
        // Get list of neighboring empty locations.
        ArrayList emptyNbrs = emptyNeighbors();
        Debug.print("Possible new locations are: " + emptyNbrs.toString());

        // If there are no valid empty neighboring locations, then we're done.
        if ( emptyNbrs.size() == 0 )
            return location();

        // Return a randomly chosen neighboring empty location.
        Random randNumGen = RandNumGenerator.getInstance();
        int randNum = randNumGen.nextInt(emptyNbrs.size());
     return (Location) emptyNbrs.get(randNum);
    }
  
  /**
   * Method to try to move this object
   */
  /** Moves this mouse in its environment.
     **/
    protected void move()
    {
        // Find a location to move to.
        Debug.print("Mouse " + toString() + " attempting to move.  ");
        Location nextLoc = nextLocation();
        
        // if the next location is the exit leave
        if (nextLoc.row() == 0 && 
            nextLoc.col() == environment().numCols() - 1)
        {
          environment().remove(this);
          System.out.println("It took " + getClass() + " " +
                             id() + " " +  numSteps + 
                             " steps to find the exit");
        }

        // If the next location is different, move there.
        else if ( ! nextLoc.equals(location()) )
        {
            // Move to new location.
            Location oldLoc = location();
            changeLocation(nextLoc);

            // Update direction in case fish had to turn to move.
            Direction newDir = environment().getDirection(oldLoc, nextLoc);
            changeDirection(newDir);
            Debug.println("  Moves to " + location() + direction());
        }
        else
            Debug.println("  Does not move.");
    }
}
