// AP(r) Computer Science Marine Biology Simulation:
//
// This class is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation.
//
// This class is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

import java.util.Random;
import java.util.*;
import java.awt.Color;
import java.util.ArrayList;

/**
 * Class that represents a smart mouse.  These
 * animals try to move toward the exit at the 
 * top right.
 * 
 * This class was created
 * to show how to extend the Marine Biology Case
 * Study past Fish. 
 * 
 * @author Barbara Ericson ericson@cc.gatech.edu
 * Georgia Tech Jan 2005
 */
public class SmartMouse extends Mouse
{
  //////////////// fields //////////////////////////
  private List visited = new ArrayList();
  
  
  //////////////// constructors ////////////////////
  
  /**
   * Constructor that takes an environment and a location
   * @param theEnv  the environment this object is in
   * @param theLoc  the location of this object
   */
  public SmartMouse(Environment theEnv, Location theLoc)
  {
    // call the other constructor with a random direction
    super(theEnv,theLoc,theEnv.randomDirection());
  }
  
  /**
   * Constructor that takes an environment, location, and 
   * direction
   * @param theEnv  the environment this object is in
   * @param theLoc  the location of this object
   * @param theDir the direction this object is facing
   */
  public SmartMouse(Environment theEnv, Location theLoc, 
                  Direction theDir)
  {
    // let the parent constructor handle this
    super(theEnv,theLoc,theDir);
  }
  
  /**
   * Constructor that takes the environment, 
   * location, direction and color
   * @param theEnv the environment to use
   * @param theLoc the location to use
   * @param theDir the direction to use
   * @param theColor the color to use
   */
  public SmartMouse(Environment theEnv, Location theLoc, 
                  Direction theDir, Color theColor)
  {
    // let the parent constructor handle this
    super(theEnv,theLoc,theDir, theColor);
  }
  
  //////////////// methods /////////////////////////
  
  /** Finds this animal's next location.
    * the smart mouse tries to move in the direction
    * of the goal (top right)
     *  @return    the next location for this fish
     **/
    protected Location nextLocation()
    {
        // Get list of neighboring empty locations.
        ArrayList emptyNbrs = emptyNeighbors();
        Debug.print("Possible new locations are: " + emptyNbrs.toString());

        // If there are no valid empty neighboring locations, then we're done.
        if ( emptyNbrs.size() == 0 )
            return location();
        
        // if one of the empty locations is to
        // the north east take that one else
        // try east, else random
        Location temp = getEmptyNeighbor(Direction.NORTHEAST);
        if (emptyNbrs.contains(temp) && !visited.contains(temp))
          return temp;
        else 
        {
          temp = getEmptyNeighbor(Direction.EAST);
          if (emptyNbrs.contains(temp) && !visited.contains(temp))
              return temp;
        }

        // Return a randomly chosen neighboring empty location.
        Random randNumGen = RandNumGenerator.getInstance();
        int randNum = randNumGen.nextInt(emptyNbrs.size());
     return (Location) emptyNbrs.get(randNum);
    }
    
    protected void move()
    {
      super.move();
      visited.add(location());
    }
}
